@@ -194,13 +194,12 @@ static uint8_t kbd_get_leds(SpiceKbdInstance *sin)
     return sif->get_leds(sin);
 }
 
-static RedPipeItem *red_inputs_key_modifiers_item_new(
-    RedChannelClient *rcc, void *data, int num)
+static RedPipeItem *red_inputs_key_modifiers_item_new(uint8_t modifiers)
 {
     RedKeyModifiersPipeItem *item = spice_malloc(sizeof(RedKeyModifiersPipeItem));
 
     red_pipe_item_init(&item->base, RED_PIPE_ITEM_KEY_MODIFIERS);
-    item->modifiers = *(uint8_t *)data;
+    item->modifiers = modifiers;
     return &item->base;
 }
 
@@ -405,7 +404,7 @@ static bool inputs_channel_handle_message(RedChannelClient *rcc, uint16_t type,
     return TRUE;
 }
 
-static void inputs_release_keys(InputsChannel *inputs)
+void inputs_release_keys(InputsChannel *inputs)
 {
     int i;
     SpiceKbdState *st;
@@ -434,14 +433,6 @@ static void inputs_release_keys(InputsChannel *inputs)
     }
 }
 
-static void inputs_channel_on_disconnect(RedChannelClient *rcc)
-{
-    if (!rcc) {
-        return;
-    }
-    inputs_release_keys(INPUTS_CHANNEL(red_channel_client_get_channel(rcc)));
-}
-
 static void inputs_pipe_add_init(RedChannelClient *rcc)
 {
     RedInputsInitPipeItem *item = spice_malloc(sizeof(RedInputsInitPipeItem));
@@ -484,8 +475,8 @@ static void inputs_channel_push_keyboard_modifiers(InputsChannel *inputs, uint8_
         inputs->src_during_migrate) {
         return;
     }
-    red_channel_pipes_new_add_push(RED_CHANNEL(inputs),
-        red_inputs_key_modifiers_item_new, (void*)&modifiers);
+    red_channel_pipes_add(RED_CHANNEL(inputs),
+                          red_inputs_key_modifiers_item_new(modifiers));
 }
 
 void inputs_channel_on_keyboard_leds_change(InputsChannel *inputs, uint8_t leds)
@@ -594,7 +585,6 @@ inputs_channel_class_init(InputsChannelClass *klass)
     channel_class->handle_message = inputs_channel_handle_message;
 
     /* channel callbacks */
-    channel_class->on_disconnect = inputs_channel_on_disconnect;
     channel_class->send_item = inputs_channel_send_item;
     channel_class->handle_migrate_data = inputs_channel_handle_migrate_data;
     channel_class->handle_migrate_flush_mark = inputs_channel_handle_migrate_flush_mark;
