@@ -61,10 +61,11 @@ RedClient *main_channel_get_client_by_link_id(MainChannel *main_chan, uint32_t l
 MainChannelClient *main_channel_link(MainChannel *, RedClient *client,
      RedsStream *stream, uint32_t link_id, int migration,
      RedChannelCapabilities *caps);
-void main_channel_push_mouse_mode(MainChannel *main_chan, int current_mode, int is_client_mouse_allowed);
+void main_channel_push_mouse_mode(MainChannel *main_chan, SpiceMouseMode current_mode,
+                                  int is_client_mouse_allowed);
 void main_channel_push_agent_connected(MainChannel *main_chan);
 void main_channel_push_agent_disconnected(MainChannel *main_chan);
-void main_channel_push_multi_media_time(MainChannel *main_chan, int time);
+void main_channel_push_multi_media_time(MainChannel *main_chan, uint32_t time);
 
 int main_channel_is_connected(MainChannel *main_chan);
 
