@@ -30,17 +30,27 @@
 #include "log.h"
 #include "backtrace.h"
 
-static int glib_debug_level = 0;
-static int abort_level = -1;
+static int glib_debug_level = INT_MAX;
+static int abort_mask = 0;
 
-#ifndef SPICE_ABORT_LEVEL_DEFAULT
+#ifndef SPICE_ABORT_MASK_DEFAULT
 #ifdef SPICE_DISABLE_ABORT
-#define SPICE_ABORT_LEVEL_DEFAULT -1
+#define SPICE_ABORT_MASK_DEFAULT 0
 #else
-#define SPICE_ABORT_LEVEL_DEFAULT SPICE_LOG_LEVEL_CRITICAL
+#define SPICE_ABORT_MASK_DEFAULT (G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_ERROR)
 #endif
 #endif
 
+#define G_LOG_DOMAIN "Spice"
+
+typedef enum {
+    SPICE_LOG_LEVEL_ERROR,
+    SPICE_LOG_LEVEL_CRITICAL,
+    SPICE_LOG_LEVEL_WARNING,
+    SPICE_LOG_LEVEL_INFO,
+    SPICE_LOG_LEVEL_DEBUG,
+} SpiceLogLevel;
+
 static GLogLevelFlags spice_log_level_to_glib(SpiceLogLevel level)
 {
     static const GLogLevelFlags glib_levels[] = {
@@ -58,7 +68,7 @@ static GLogLevelFlags spice_log_level_to_glib(SpiceLogLevel level)
 
 static void spice_log_set_debug_level(void)
 {
-    if (glib_debug_level == 0) {
+    if (glib_debug_level == INT_MAX) {
         const char *debug_str = g_getenv("SPICE_DEBUG_LEVEL");
         if (debug_str != NULL) {
             int debug_level;
@@ -83,10 +93,10 @@ static void spice_log_set_debug_level(void)
              */
             debug_env = (char *)g_getenv("G_MESSAGES_DEBUG");
             if (debug_env == NULL) {
-                g_setenv("G_MESSAGES_DEBUG", SPICE_LOG_DOMAIN, FALSE);
+                g_setenv("G_MESSAGES_DEBUG", G_LOG_DOMAIN, FALSE);
             } else {
-                debug_env = g_strconcat(debug_env, ":", SPICE_LOG_DOMAIN, NULL);
-                g_setenv("G_MESSAGES_DEBUG", SPICE_LOG_DOMAIN, FALSE);
+                debug_env = g_strconcat(debug_env, " ", G_LOG_DOMAIN, NULL);
+                g_setenv("G_MESSAGES_DEBUG", G_LOG_DOMAIN, FALSE);
                 g_free(debug_env);
             }
         }
@@ -95,25 +105,23 @@ static void spice_log_set_debug_level(void)
 
 static void spice_log_set_abort_level(void)
 {
-    if (abort_level == -1) {
+    if (abort_mask == 0) {
         const char *abort_str = g_getenv("SPICE_ABORT_LEVEL");
         if (abort_str != NULL) {
             GLogLevelFlags glib_abort_level;
 
             /* FIXME: To be removed after enough deprecation time */
             g_warning("Setting SPICE_ABORT_LEVEL is deprecated, use G_DEBUG instead");
-            abort_level = atoi(abort_str);
-            glib_abort_level = spice_log_level_to_glib(abort_level);
-            if (glib_abort_level != 0) {
-                unsigned int fatal_mask = G_LOG_FATAL_MASK;
-                while (glib_abort_level >= G_LOG_LEVEL_ERROR) {
-                    fatal_mask |= glib_abort_level;
-                    glib_abort_level >>= 1;
-                }
-                g_log_set_fatal_mask(SPICE_LOG_DOMAIN, fatal_mask);
+            glib_abort_level = spice_log_level_to_glib(atoi(abort_str));
+            unsigned int fatal_mask = G_LOG_FATAL_MASK;
+            while (glib_abort_level >= G_LOG_LEVEL_ERROR) {
+                fatal_mask |= glib_abort_level;
+                glib_abort_level >>= 1;
             }
+            abort_mask = fatal_mask;
+            g_log_set_fatal_mask(G_LOG_DOMAIN, fatal_mask);
         } else {
-            abort_level = SPICE_ABORT_LEVEL_DEFAULT;
+            abort_mask = SPICE_ABORT_MASK_DEFAULT;
         }
     }
 }
@@ -123,9 +131,8 @@ static void spice_logger(const gchar *log_domain,
                          const gchar *message,
                          gpointer user_data G_GNUC_UNUSED)
 {
-    if (glib_debug_level != 0) {
-        if ((log_level & G_LOG_LEVEL_MASK) > glib_debug_level)
-            return; // do not print anything
+    if ((log_level & G_LOG_LEVEL_MASK) > glib_debug_level) {
+        return; // do not print anything
     }
     g_log_default_handler(log_domain, log_level, message, NULL);
 }
@@ -135,7 +142,7 @@ SPICE_CONSTRUCTOR_FUNC(spice_log_init)
 
     spice_log_set_debug_level();
     spice_log_set_abort_level();
-    g_log_set_handler(SPICE_LOG_DOMAIN,
+    g_log_set_handler(G_LOG_DOMAIN,
                       G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
                       spice_logger, NULL);
     /* Threading is always enabled from 2.31.0 onwards */
@@ -149,7 +156,7 @@ SPICE_CONSTRUCTOR_FUNC(spice_log_init)
 }
 
 static void spice_logv(const char *log_domain,
-                       SpiceLogLevel log_level,
+                       GLogLevelFlags log_level,
                        const char *strloc,
                        const char *function,
                        const char *format,
@@ -157,7 +164,9 @@ static void spice_logv(const char *log_domain,
 {
     GString *log_msg;
 
-    g_return_if_fail(spice_log_level_to_glib(log_level) != 0);
+    if ((log_level & G_LOG_LEVEL_MASK) > glib_debug_level) {
+        return; // do not print anything
+    }
 
     log_msg = g_string_new(NULL);
     if (strloc && function) {
@@ -166,17 +175,16 @@ static void spice_logv(const char *log_domain,
     if (format) {
         g_string_append_vprintf(log_msg, format, args);
     }
-    g_log(log_domain, spice_log_level_to_glib(log_level), "%s", log_msg->str);
+    g_log(log_domain, log_level, "%s", log_msg->str);
     g_string_free(log_msg, TRUE);
 
-    if (abort_level != -1 && abort_level >= (int) log_level) {
+    if ((abort_mask & log_level) != 0) {
         spice_backtrace();
         abort();
     }
 }
 
-void spice_log(const char *log_domain,
-               SpiceLogLevel log_level,
+void spice_log(GLogLevelFlags log_level,
                const char *strloc,
                const char *function,
                const char *format,
@@ -185,6 +193,6 @@ void spice_log(const char *log_domain,
     va_list args;
 
     va_start (args, format);
-    spice_logv (log_domain, log_level, strloc, function, format, args);
+    spice_logv (G_LOG_DOMAIN, log_level, strloc, function, format, args);
     va_end (args);
 }
