@@ -1,4 +1,4 @@
-import codegen
+from . import codegen
 import types
 
 _types_by_name = {}
@@ -7,7 +7,7 @@ _types = []
 default_pointer_size = 4
 
 def type_exists(name):
-    return _types_by_name.has_key(name)
+    return name in _types_by_name
 
 def lookup_type(name):
     return _types_by_name[name]
@@ -24,7 +24,7 @@ class FixedSize:
             self.vals[minor] = val
 
     def __add__(self, other):
-        if isinstance(other, types.IntType):
+        if isinstance(other, int):
             other = FixedSize(other)
 
         new = FixedSize()
@@ -62,10 +62,84 @@ class FixedSize:
 # other members
 propagated_attributes=["ptr_array", "nonnull", "chunk"]
 
+valid_attributes=set([
+    # embedded/appended at the end of the resulting C structure
+    'end',
+    # the C structure contains a pointer to data
+    # for instance we want to write an array to an allocated array
+    'to_ptr',
+    # write output to this C structure
+    'ctype',
+    # prefix for flags/values enumerations
+    'prefix',
+    # used in demarshaller to use directly data from message without a copy
+    'nocopy',
+    # store member array in a pointer
+    # similar to to_ptr but has an additional argument which is the name of a C
+    # field which will store the array length
+    'as_ptr',
+    # do not generate marshall code
+    # used for last members to be able to marshall them manually
+    'nomarshal',
+    # ??? not used by python code
+    'zero_terminated',
+    'marshall',
+    # this pointer member cannot be null
+    'nonnull',
+    # this flag member contains only a single flag
+    'unique_flag',
+    'ptr_array',
+    'outvar',
+    # C structure has an anonymous member (used in switch)
+    'anon',
+    'chunk',
+    # this channel is contained in an #ifdef section
+    # the argument specifies the preprocessor define to check
+    'ifdef',
+    # write this member as zero on network
+    'zero',
+    # specify minor version required for these members
+    'minor',
+    # this member contains the byte count for an array.
+    # the argument is the member name for item count (not bytes)
+    'bytes_count',
+    # this attribute does not exist on the network, fill just structure with the value
+    'virtual',
+    # for a switch this indicates that on network
+    # it will occupy always the same size (maximum size required for all members)
+    'fixedsize',
+])
+
+attributes_with_arguments=set([
+    'ctype',
+    'prefix',
+    'as_ptr',
+    'outvar',
+    'ifdef',
+    'minor',
+    'bytes_count',
+    'virtual',
+])
+
+def fix_attributes(attribute_list):
+    attrs = {}
+    for attr in attribute_list:
+        name = attr[0][1:]
+        lst = attr[1:]
+        if not name in valid_attributes:
+            raise Exception("Attribute %s not recognized" % name)
+        if not name in attributes_with_arguments:
+            if len(lst) > 0:
+                raise Exception("Attribute %s specified with options" % name)
+        elif len(lst) > 1:
+            raise Exception("Attribute %s has more than 1 argument" % name)
+        attrs[name] = lst
+    return attrs
+
 class Type:
     def __init__(self):
         self.attributes = {}
-        self.registred = False
+        self.registered = False
         self.name = None
 
     def has_name(self):
@@ -122,16 +196,18 @@ class Type:
         return self
 
     def register(self):
-        if self.registred or self.name == None:
+        if self.registered or self.name == None:
             return
-        self.registred = True
-        if _types_by_name.has_key(self.name):
-            raise Exception, "Type %s already defined" % self.name
+        self.registered = True
+        if self.name in _types_by_name:
+            raise Exception("Type %s already defined" % self.name)
         _types.append(self)
         _types_by_name[self.name] = self
 
     def has_attr(self, name):
-        return self.attributes.has_key(name)
+        if not name in valid_attributes:
+            raise Exception('attribute %s not expected' % name)
+        return name in self.attributes
 
 class TypeRef(Type):
     def __init__(self, name):
@@ -142,12 +218,12 @@ class TypeRef(Type):
         return "ref to %s" % (self.name)
 
     def resolve(self):
-        if not _types_by_name.has_key(self.name):
-            raise Exception, "Unknown type %s" % self.name
+        if self.name not in _types_by_name:
+            raise Exception("Unknown type %s" % self.name)
         return _types_by_name[self.name]
 
     def register(self):
-        assert True, "Can't register TypeRef!"
+        assert False, "Can't register TypeRef!"
 
 
 class IntegerType(Type):
@@ -168,7 +244,7 @@ class IntegerType(Type):
         return self.name + "_t"
 
     def get_fixed_nw_size(self):
-        return self.bits / 8
+        return self.bits // 8
 
     def is_primitive(self):
         return True
@@ -178,8 +254,7 @@ class TypeAlias(Type):
         Type.__init__(self)
         self.name = name
         self.the_type = the_type
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def get_type(self, recursive=False):
         if recursive:
@@ -245,7 +320,24 @@ class EnumBaseType(Type):
         return True
 
     def get_fixed_nw_size(self):
-        return self.bits / 8
+        return self.bits // 8
+
+    # generates a value-name table suitable for use with the wireshark protocol
+    # dissector
+    def c_describe(self, writer):
+        writer.write("static const value_string %s_vs[] = " % codegen.prefix_underscore_lower(self.name))
+        writer.begin_block()
+        values = list(self.names.keys())
+        values.sort()
+        for i in values:
+            writer.write("{ ")
+            writer.write(self.c_enumname(i))
+            writer.write(", \"%s\" }," % self.names[i])
+            writer.newline()
+        writer.write("{ 0, NULL }")
+        writer.end_block(semicolon=True)
+        writer.newline()
+
 
 class EnumType(EnumBaseType):
     def __init__(self, bits, name, enums, attribute_list):
@@ -264,15 +356,14 @@ class EnumType(EnumBaseType):
                 value = last + 1
             last = value
 
-            assert not names.has_key(value)
+            assert value not in names
             names[value] = name
             values[name] = value
 
         self.names = names
         self.values = values
 
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def __str__(self):
         return "enum %s" % self.name
@@ -281,7 +372,7 @@ class EnumType(EnumBaseType):
         writer.write("typedef enum ")
         writer.write(self.c_name())
         writer.begin_block()
-        values = self.names.keys()
+        values = list(self.names.keys())
         values.sort()
         current_default = 0
         for i in values:
@@ -318,15 +409,14 @@ class FlagsType(EnumBaseType):
                 value = last + 1
             last = value
 
-            assert not names.has_key(value)
+            assert value not in names
             names[value] = name
             values[name] = value
 
         self.names = names
         self.values = values
 
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def __str__(self):
         return "flags %s" % self.name
@@ -335,7 +425,7 @@ class FlagsType(EnumBaseType):
         writer.write("typedef enum ")
         writer.write(self.c_name())
         writer.begin_block()
-        values = self.names.keys()
+        values = list(self.names.keys())
         values.sort()
         mask = 0
         for i in values:
@@ -375,26 +465,26 @@ class ArrayType(Type):
         return self
 
     def is_constant_length(self):
-        return isinstance(self.size, types.IntType)
+        return isinstance(self.size, int)
 
     def is_remaining_length(self):
-        return isinstance(self.size, types.StringType) and len(self.size) == 0
+        return isinstance(self.size, str) and len(self.size) == 0
 
     def is_identifier_length(self):
-        return isinstance(self.size, types.StringType) and len(self.size) > 0
+        return isinstance(self.size, str) and len(self.size) > 0
 
     def is_image_size_length(self):
-        if isinstance(self.size, types.IntType) or isinstance(self.size, types.StringType):
+        if isinstance(self.size, int) or isinstance(self.size, str):
             return False
         return self.size[0] == "image_size"
 
     def is_bytes_length(self):
-        if isinstance(self.size, types.IntType) or isinstance(self.size, types.StringType):
+        if isinstance(self.size, int) or isinstance(self.size, str):
             return False
         return self.size[0] == "bytes"
 
     def is_cstring_length(self):
-        if isinstance(self.size, types.IntType) or isinstance(self.size, types.StringType):
+        if isinstance(self.size, int) or isinstance(self.size, str):
             return False
         return self.size[0] == "cstring"
 
@@ -406,7 +496,7 @@ class ArrayType(Type):
 
     def get_fixed_nw_size(self):
         if not self.is_fixed_nw_size():
-            raise Exception, "Not a fixed size type"
+            raise Exception("Not a fixed size type")
 
         return self.element_type.get_fixed_nw_size() * self.size
 
@@ -416,13 +506,13 @@ class ArrayType(Type):
             return 0
         if self.is_constant_length(self):
             return element_count * self.size
-        raise Exception, "Pointers in dynamic arrays not supported"
+        raise Exception("Pointers in dynamic arrays not supported")
 
     def get_pointer_names(self, marshalled):
         element_count = self.element_type.get_num_pointers()
         if element_count  == 0:
             return []
-        raise Exception, "Pointer names in arrays not supported"
+        raise Exception("Pointer names in arrays not supported")
 
     def is_extra_size(self):
         return self.has_attr("ptr_array")
@@ -500,7 +590,9 @@ class Containee:
         return not self.is_switch() and self.member_type.is_primitive()
 
     def has_attr(self, name):
-        return self.attributes.has_key(name)
+        if not name in valid_attributes:
+            raise Exception('attribute %s not expected' % name)
+        return name in self.attributes
 
     def has_minor_attr(self):
         return self.has_attr("minor")
@@ -516,15 +608,12 @@ class Member(Containee):
         Containee.__init__(self)
         self.name = name
         self.member_type = member_type
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def resolve(self, container):
         self.container = container
         self.member_type = self.member_type.resolve()
         self.member_type.register()
-        if self.has_attr("ptr32") and self.member_type.is_pointer():
-            self.member_type.set_ptr_size(4)
         for i in propagated_attributes:
             if self.has_attr(i):
                 self.member_type.attributes[i] = self.attributes[i]
@@ -582,7 +671,7 @@ class Member(Containee):
             names = self.member_type.get_pointer_names(marshalled)
         if self.has_attr("outvar"):
             prefix = self.attributes["outvar"][0]
-            names = map(lambda name: prefix + "_" + name, names)
+            names = [prefix + "_" + name for name in names]
         return names
 
 class SwitchCase:
@@ -597,6 +686,7 @@ class SwitchCase:
             if v == None:
                 return "1"
             elif var_type.is_enum():
+                assert v[0] == "", "Negation of enumeration in switch is not supported"
                 checks.append("%s == %s" % (var_cname, var_type.c_enumname_by_name(v[1])))
             else:
                 checks.append("%s(%s & %s)" % (v[0], var_cname, var_type.c_enumname_by_name(v[1])))
@@ -619,8 +709,7 @@ class Switch(Containee):
         self.variable = variable
         self.name = name
         self.cases = cases
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def is_switch(self):
         return True
@@ -639,7 +728,7 @@ class Switch(Containee):
 
     def resolve(self, container):
         self.container = container
-        self.cases = map(lambda c : c.resolve(self), self.cases)
+        self.cases = [c.resolve(self) for c in self.cases]
         return self
 
     def __repr__(self):
@@ -686,7 +775,7 @@ class Switch(Containee):
 
     def get_fixed_nw_size(self):
         if not self.is_fixed_nw_size():
-            raise Exception, "Not a fixed size type"
+            raise Exception("Not a fixed size type")
         size = 0
         for c in self.cases:
             size = max(size, c.member.get_fixed_nw_size())
@@ -757,7 +846,7 @@ class ContainerType(Type):
         return size
 
     def resolve(self):
-        self.members = map(lambda m : m.resolve(self), self.members)
+        self.members = [m.resolve(self) for m in self.members]
         return self
 
     def get_num_pointers(self):
@@ -802,7 +891,7 @@ class ContainerType(Type):
             name = name[:dot]
 
         member = None
-        if self.members_by_name.has_key(name):
+        if name in self.members_by_name:
             member = self.members_by_name[name]
         else:
             for m in self.members:
@@ -814,7 +903,7 @@ class ContainerType(Type):
                     break
 
         if member == None:
-            raise Exception, "No member called %s found" % name
+            raise Exception("No member called %s found" % name)
 
         if rest != None:
             return member.member_type.lookup_member(rest)
@@ -829,8 +918,7 @@ class StructType(ContainerType):
         self.members_by_name = {}
         for m in members:
             self.members_by_name[m.name] = m
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def __str__(self):
         if self.name == None:
@@ -852,8 +940,7 @@ class MessageType(ContainerType):
         for m in members:
             self.members_by_name[m.name] = m
         self.reverse_members = {} # ChannelMembers referencing this message
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def __str__(self):
         if self.name == None:
@@ -863,7 +950,7 @@ class MessageType(ContainerType):
 
     def c_name(self):
         if self.name == None:
-            cms = self.reverse_members.keys()
+            cms = list(self.reverse_members.keys())
             if len(cms) != 1:
                 raise "Unknown typename for message"
             cm = cms[0]
@@ -883,7 +970,7 @@ class MessageType(ContainerType):
         if self.has_attr("ctype"):
             return self.attributes["ctype"][0]
         if self.name == None:
-            cms = self.reverse_members.keys()
+            cms = list(self.reverse_members.keys())
             if len(cms) != 1:
                 raise "Unknown typename for message"
             cm = cms[0]
@@ -921,8 +1008,7 @@ class ChannelType(Type):
         self.base = base
         self.member_name = None
         self.members = members
-        for attr in attribute_list:
-            self.attributes[attr[0][1:]] = attr[1:]
+        self.attributes = fix_attributes(attribute_list)
 
     def __str__(self):
         if self.name == None:
@@ -1034,6 +1120,14 @@ class ProtocolType(Type):
 
         return self
 
+class FdType(IntegerType):
+    def __init__(self):
+        IntegerType.__init__(self, 0, False)
+        self.name = "fd"
+
+    def c_type(self):
+        return "int"
+
 int8 = IntegerType(8, True)
 uint8 = IntegerType(8, False)
 int16 = IntegerType(16, True)
@@ -1042,3 +1136,4 @@ int32 = IntegerType(32, True)
 uint32 = IntegerType(32, False)
 int64 = IntegerType(64, True)
 uint64 = IntegerType(64, False)
+unix_fd = FdType()
